;; This locales.clj sample is deliberately conflicting with the de-ru-es.clj.
;; It is used by the tests in core_test.clj.
{
  :locales #{"de"}
  :package "example.i18n"
  :bundle  "alternate.i18n.Messages"
}
