;; This is a sample of the locales.clj file that the old version of
;; the Makefile used to generate.
;; It is used by the tests in core_test.clj.
{
  :locales #{"it" "fr"}
  :package "other.i18n"
  :bundle  "other.i18n.Messages"
}
